<?php
namespace Ep\Woo\Product;

/**
 * Class Simple
 *
 * @package Ep\Woo\Entity\Product
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Simple extends \WC_Product_Simple implements ProductInterface
{
    use ProductTrait;

    /** @var array */
    protected $extra_data = [
        'price_hidden' => false,
        'purchase_step' => 1,
        'brand' => []
    ];

    /**
     * Returns the availability of the product.
     *
     * @return string[]
     */
    public function get_availability()
    {
        if ($this->is_price_hidden()) {
            return [];
        }

        return apply_filters('woocommerce_get_availability', [
            'availability' => $this->get_availability_text(),
            'class'        => $this->get_availability_class(),
        ], $this);
    }

    /**
     * Najveća količina ovog proizvoda koju je moguće kupiti.
     *
     * @return int
     */
    public function get_max_purchase_quantity()
    {
        if (0 === floatval($this->get_price(null))) {
            return 0;
        }

        $max = parent::get_max_purchase_quantity();
        $step = intval($this->get_purchase_step());

        if ($step > $max) {
            return 0;
        }

        $max = floor($max / $step) * $step;

        return $max < $step ? 0 : $max;
    }

    /**
     * Najmanja količina ovog proizvoda koju je moguće kupiti.
     *
     * @return int
     */
    public function get_min_purchase_quantity()
    {
        $min = parent::get_min_purchase_quantity();

        return $this->get_max_purchase_quantity() > 0
            ? max($min, $this->get_purchase_step())
            : 1;
    }

    /**
     * Returns the product's active price.
     *
     * @param  string $context What the value is for. Valid values are view and edit.
     * @return string price
     */
    public function get_price($context = 'view')
    {
        if ($this->is_price_hidden()) {
            return '';
        }

        $price = parent::get_price($context);
        return $price === '0' ? '' : $price;
    }

    public function get_regular_price( $context = 'view' )
    {
        if ($this->is_price_hidden()) {
            return '';
        }

        $price = parent::get_regular_price($context);
        return $price === '0' ? '' : $price;
    }

    /**
     * Return the stock status.
     *
     * @param  string $context What the value is for. Valid values are view and edit.
     * @since 3.0.0
     * @return string
     */
    public function get_stock_status($context = 'view')
    {
        return $this->get_max_purchase_quantity() === 0
            ? 'outofstock'
            : $this->get_prop('stock_status', $context);
    }

    /**
     * Returns whether or not the product can be purchased.
     * This returns true for 'instock' and 'onbackorder' stock statuses.
     *
     * @return bool
     */
    public function is_in_stock()
    {
        return apply_filters('woocommerce_product_is_in_stock', 'outofstock' !== $this->get_stock_status(), $this);
    }

    public function get_price_hidden()
    {
        return $this->is_price_hidden();
    }

    public function get_brand()
    {
        return $this->get_prop('brand');
    }

    /**
     * Da li je cijena ovog proizvoda skrivena.
     *
     * @return mixed
     */
    public function is_price_hidden()
    {
        return $this->get_prop('price_hidden', null);
    }

    /**
     * Da li je ovaj proizvod moguće kupiti.
     *
     * @return bool
     */
    public function is_purchasable()
    {
        return
            parent::is_purchasable() &&
            $this->get_min_purchase_quantity() > 0 &&
            $this->get_max_purchase_quantity() > 0;
    }

    /**
     * Set gallery attachment ids.
     *
     * @since 3.0.0
     * @param array $image_ids List of image ids.
     */
    public function set_gallery_image_ids($image_ids)
    {
        $this->set_prop('gallery_image_ids', $image_ids);
    }

    /**
     * Postavlja da li je cije na ovog proizvoda skrivena.
     *
     * @param $price_hidden
     */
    public function set_price_hidden($price_hidden)
    {
        $this->set_prop('price_hidden', $price_hidden);
    }

    public function set_brand( $brand )
    {
        $this->set_prop('brand', $brand);
    }

    /**
     * Postavlja korake u kojima je moguće naručiti proizvod.
     *
     * @param int $step
     */
    public function set_purchase_step($step)
    {
        $this->set_prop('purchase_step', $step);
    }
}